<div
    id="{{ widget_id }}-container"
    class="newsflash-widget-container"
    hx-post="{{ full_path }}"
    hx-trigger="revealed delay:50ms, resizeChart delay:250ms"
    hx-swap="none"
    hx-include="{{ hx_include|join(', ') }}"
>
    <svg 
        xmlns="http://www.w3.org/2000/svg" 
        id="{{ widget_id }}">
    </svg>
</div>