<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Document</title>

    <!-- <link rel="preconnect" href="https://fonts.googleapis.com">
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
    <link href="https://fonts.googleapis.com/css2?family=Noto+Serif:ital,wght@0,100..900;1,100..900&display=swap" rel="stylesheet"> -->

    <link rel="preconnect" href="https://fonts.googleapis.com">
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
    <link href="https://fonts.googleapis.com/css2?family=Lora:ital,wght@0,400..700;1,400..700&display=swap" rel="stylesheet">

    <style>
        body {
            font-family: 'Lora', serif;
            background-color: #1e1e2e;
            color: #cdd6f4;
        }

        .white {
            fill: #cdd6f4;   
        }

        .stroke-white {
            stroke: #45475a;
        }

        .stroke-width-2 {
            stroke-width: 2;
        }

        .stroke-rounded {
            stroke-linecap: round;
        }

        .blue {
            fill: #89b4fa;
        }

        .lightgray {
            fill: lightgray;
        }

        .lightblue {
            fill: lightblue;
        }
    </style>
</head>
<body>
    {{ content | safe }}
</body>
</html>