<ul
    id="{{ id }}"
    class="newsflash-list"
    {% if hx_swap_oob %}hx-swap-oob="true"{% endif %}
>
    <input type="hidden" name="{{ id }}-id" value="{{ id }}">
    {% for item_id, rendered_item in widgets.items() %}
        <li
            class="newsflash-list-item"
        >
            <input type="hidden" name="{{ id }}-{{ loop.index0 }}-id" value="{{ item_id }}">
            {{ rendered_item | safe }}
        </li>
    {% endfor %}
</ul>