body {
    background-color: white;
    color: black;
    font-family: sans-serif;
    font-size: 18px;
}

h1 {
    font-size: 40px;
    font-weight: 400;
    color: hsl(0, 0%, 20%);
}

h2 {
    font-size: 30px;
    font-weight: 400;
    margin: 0;
}

.text {
    fill: hsl(0, 0%, 0%);
}

.grid-line {
    stroke: hsl(0, 0%, 40%);
    stroke-width: 0.5;
    stroke-linecap: round;
}

.newsflash-input {
    font-family: 'Lora', serif;
    font-size: 18px;
    color: hsl(0, 0%, 0%);
    background-color: hsl(233, 34%, 66%);
    outline: none;
    border-radius: 0.25em;
    border-style: solid;
    border-width: 2px;
    border-color: hsl(232, 27%, 70%);
    padding: 0.75em 1.1em;
    margin: 0;
    width: 100%;
    box-sizing: border-box;
}

.newsflash-input:focus {
    border-color: hsl(232, 27%, 45%);
    /* border-style: dashed; */
}

.newsflash-textarea {
    resize: none;
}

.newsflash-button {
    font-family: 'Lora', serif;
    font-size: 18px;
    /* color: #cdd6f4; */
    color: white;
    /* background-color: hsl(232, 37%, 35%); */
    background-color: hsl(135, 10%, 30%);
    border-radius: 0.25em;
    padding: 0.75em 1.1em;
    width: 100%;
    box-sizing: border-box;
    border-style: solid;
    outline: none;
    border-width: 2px;
    border-color: hsl(135, 10%, 30%);
    box-shadow: 4px 4px 0 hsl(0, 0%, 60%);
    transform: translate(-4px, -4px);
    transition: box-shadow 100ms, transform 100ms;
}

.newsflash-button:hover {
    box-shadow: 6px 6px 0 hsl(0, 0%, 60%);
    transform: translate(-6px, -6px);
    cursor: pointer;
}

.newsflash-button:active {
    box-shadow: 2px 2px 0 hsl(0, 0%, 60%);
    transform: translate(-2px, -2px);
}

.newsflash-button:disabled {
    cursor: default;
    /* background-color: hsl(232, 20%, 35%); */
    background-color: hsl(28, 0%, 40%);
    border-color: hsl(28, 0%, 40%);
    /* border-color: hsl(232, 20%, 35%); */
}

.newsflash-button:disabled:hover, .newsflash-button:disabled:focus {
    cursor: default;
    box-shadow: 4px 4px 0 hsl(0, 0%, 60%);
    transform: translate(-4px, -4px);
}

.newsflash-widget-container {
    position: relative;
    height: 100%;
    width: 100%;
}

.bar {
    opacity: 0;
    fill: hsl(0, 65%, 35%);
    transform: scaleY(0);
    transform-origin: bottom;
    transform-box: fill-box;
    animation: drawBar 400ms forwards;
}

@keyframes drawBar {
    to {
        opacity: 1;
        transform: scaleY(1);
    }
}

@keyframes fadeIn {
    from {
        opacity: 0;
    }
    to {
        opacity: 1;
    }
}

.line-path {
    fill: transparent;
    /* stroke: hsl(217, 90%, 75%); */
    stroke: hsl(0, 65%, 35%);
    stroke-width: 4;
    stroke-linecap: round;
    stroke-dasharray: 100 200;
    stroke-dashoffset: 100;
    animation: drawLine 400ms forwards;
}

@keyframes drawLine {
    to {
        stroke-dasharray: 100 200;
        stroke-dashoffset: 0;
    }
}

.newsflash-dropdown-container {
    position: relative;
}

.newsflash-dropdown-input {
    cursor: pointer;
}

.newsflash-dropdown-list {
    position: absolute;
    width: 100%;
    list-style-type: none;
    margin: 0.5em 0 0 0;
    padding: 0;
    max-height: 40vh;
    /* background-color: hsl(232, 24%, 22%); */
    background-color: hsl(28, 100%, 90%);
    /* background-color: hsl(28, 60%, 90%); */
    overflow-y: auto;
    box-sizing: border-box;
    border-radius: 0.25em;
    border-style: solid;
    border-width: 2px;
    border-color: hsl(232, 27%, 45%);
    /* box-shadow: 0 14px 30px #181825; */
    /* box-shadow: 5px 5px 0 hsl(0, 0%, 60%); */
    border-radius: 0.25em;
    z-index: 10;
}

.newsflash-dropdown-option {
    padding: 0.75em 1.1em;
    cursor: pointer;
}

.newsflash-dropdown-option:hover {
    /* background-color: hsl(232, 28%, 32%); */
    background-color: hsl(213, 44%, 85%);
}

.newsflash-value-display-label {
    font-size: 16px;
    /* color: rgba(180, 190, 254, 0.6); */
    color: hsl(0, 0%, 40%);
    /* text-decoration: underline; */
    margin: 0;
}

.newsflash-value-display-value {
    font-size: 28px;
    margin: 0.15em 0;
}

#newsflash-notifications-container {
    position: fixed;
    right: 1rem;
    bottom: 1rem;
    z-index: 1000;
    width: 500px;
}

ul {
    padding: 0;
}

li {
    list-style-type: none;
}

.newsflash-notification {
    /* background-color: hsl(232, 24%, 22%); */
    background-color: hsl(213, 44%, 80%);
    color: black;
    padding: 0.5rem 1.5rem;
    list-style-type: none;
    margin-top: 1rem;
    /* border-radius: 0.25rem; */
    /* box-shadow: 0 0 40px hsl(240, 21%, 3%); */
    box-shadow: 0 5px 5px hsl(228, 10%, 10%);
    border-width: 1px;
    border-style: solid;
    border-color: hsl(0, 0%, 30%);
}

.notification-out {
    animation: notificationOut 1000ms forwards;
}

@keyframes notificationOut {
    to {
        transform: translateX(120%);
    }
}
    
.newsflash-chart {
    height: 400px;
    /* background-color: hsl(0, 32%, 70%); */
    /* background-color: #1e1e2e; */
    padding: 1em;
    border-radius: 0.25em;
}

/* @media (prefers-color-scheme: dark) {
    body {
        background-color: #181825;
        color: hsl(226, 64%, 94%);
    }

    h1, h2 {
        color: hsl(226, 54%, 80%);
    }

    .text {
        fill: hsl(226, 64%, 94%);
    }

    .grid-line {
        stroke: hsl(226, 64%, 94%);
    }

    .newsflash-input {
        color: #cdd6f4;
        background-color: #1e1e2e;
        border-color: hsl(236, 10%, 28%);
    }

    .newsflash-input:focus {
        border-color: hsl(232, 27%, 45%);
    }

    .line-path {
        stroke: hsl(217, 90%, 75%);
    }
} */